// Copyright (C) 2009 - CEA - Jean-Marc Martinez
// Copyright (C) 2009-2011 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

//
// Compute the first order sensitivity indices of the ishigami function.
// Uses the Sobol, Homma, Saltelli method.
// Three random variables uniform in [-pi,pi].
// Uses Monte-Carlo experiments to compute the sensitivity indices.
//

tic();

function y = ishigami (x)
  // Returns the output y of the product x1 * x2.
  // Parameters
  // x: a np-by-nx matrix of doubles, where np is the number of experiments, and nx=2.
  // y: a np-by-1 matrix of doubles
  a=7.
  b=0.1
  s1=sin(x(:,1))
  s2=sin(x(:,2))
  x34 = x(:,3).^4
  y(:,1) = s1 + a.*s2.^2 + b.*x34.*s1
endfunction

function s = sensitivityindex(ya,yc)
  // Returns the sensitivity index associated with experiments ya and yc.
  C = cov (ya, yc)
  s= C(1,2)/ (st_deviation(ya) * st_deviation(yc))
endfunction

function exact = ishigami_saexact ( a , b )
    // Exact results for the Ishigami function
    exact.expectation = a/2;
    exact.var = 1/2 + a^2/8 + b*%pi^4/5 + b^2*%pi^8/18;
    // Sensitivity indices.
    exact.S1 = (1/2 + b*%pi^4/5+b^2*%pi^8/50)/exact.var;
    exact.S2 = (a^2/8)/exact.var;
    exact.S3 = 0;
    exact.S12 = 0;
    exact.S23 = 0;
    exact.S13 = b^2*%pi^8/2*(1/9-1/25)/exact.var;
    exact.S123 = 0;
    exact.ST1 = exact.S1 + exact.S12 + exact.S13 + exact.S123;
    exact.ST2 = exact.S2 + exact.S12 + exact.S23 + exact.S123;
    exact.ST3 = exact.S3 + exact.S13 + exact.S23 + exact.S123;
endfunction

// Exact results
a=7.;
b=0.1;
exact = ishigami_saexact ( a , b );

// Create the uncertain parameters
rvu1 = randvar_new("Uniforme",-%pi,%pi);
rvu2 = randvar_new("Uniforme",-%pi,%pi);
rvu3 = randvar_new("Uniforme",-%pi,%pi);
srvu = setrandvar_new();
setrandvar_addrandvar ( srvu, rvu1);
setrandvar_addrandvar ( srvu, rvu2);
setrandvar_addrandvar ( srvu, rvu3);
// The number of uncertain parameters is :
nx = setrandvar_getdimension(srvu);
np = 10000;
// Create a first sampling A
setrandvar_buildsample(srvu,"Lhs",np);
A = setrandvar_getsample(srvu);
// Create a first sampling B
setrandvar_buildsample(srvu,"Lhs",np);
B = setrandvar_getsample(srvu);
// Perform the experiments in A
ya = ishigami(A);
// Compute the first order sensitivity index for X1
C = B;
C(:,1)=A(:,1);
yc = ishigami(C);
s1 = sensitivityindex(ya,yc);
mprintf("S1 : %f (expected = %f)\n", s1, exact.S1);
// Compute the first order sensitivity index for X2
C = B;
C(:,2)=A(:,2);
yc = ishigami(C);
s2 = sensitivityindex(ya,yc);
mprintf("S2 : %f (expected = %f)\n", s2, exact.S2);
// Compute the first order sensitivity index for X3
C = B;
C(:,3)=A(:,3);
yc = ishigami(C);
s3 = sensitivityindex(ya,yc);
mprintf("S3 : %f (expected = %f)\n", s3, exact.S3);
// Compute the sensitivity index for {X1,X2}
C = B;
C(:,[1 2])=A(:,[1 2]);
yc = ishigami(C);
s12 = sensitivityindex(ya,yc)-s1-s2;
mprintf("S12 : %f (expected = %f)\n", s12, exact.S12);
// Compute the sensitivity index for {X1,X3}
C = B;
C(:,[1 3])=A(:,[1 3]);
yc = ishigami(C);
s13 = sensitivityindex(ya,yc)-s1-s3;
mprintf("S13 : %f (expected = %f)\n", s13, exact.S13);
// Compute the sensitivity index for {X2,X3}
C = B;
C(:,[2 3])=A(:,[2 3]);
yc = ishigami(C);
s23 = sensitivityindex(ya,yc)-s2-s3;
mprintf("S23 : %f (expected = %f)\n", s23, exact.S23);
// Compute the sensitivity index for {X1,X2,X3}
C = B;
C(:,[1 2 3])=A(:,[1 2 3]);
yc = ishigami(C);
s123 = sensitivityindex(ya,yc)-s1-s2-s3-s12-s23-s13;
mprintf("S123 : %f (expected = %f)\n", s123, exact.S123);
// Compute the total sensitivity index for X1
C = A;
C(:,1)=B(:,1);
yc = ishigami(C);
st1 = 1-sensitivityindex(ya,yc);
mprintf("ST1 : %f (expected = %f)\n", st1, exact.ST1);
// Compute the total sensitivity index for X2
C = A;
C(:,2)=B(:,2);
yc = ishigami(C);
st2 = 1-sensitivityindex(ya,yc);
mprintf("ST2 : %f (expected = %f)\n", st2, exact.ST2);
// Compute the total sensitivity index for X3
C = A;
C(:,3)=B(:,3);
yc = ishigami(C);
st3 = 1-sensitivityindex(ya,yc);
mprintf("ST3 : %f (expected = %f)\n", st3, exact.ST3);

//
// Clean-up
randvar_destroy(rvu1);
randvar_destroy(rvu2);
randvar_destroy(rvu3);
setrandvar_destroy(srvu);

t = toc();
mprintf("Time = %f (s)\n", t);
//
// Load this script into the editor
//
filename = 'ishigami_hommasaltelli.sce';
dname = get_absolute_file_path(filename);
editor ( dname + filename );

