// Copyright (C) 2013 - Michael Baudin
// Copyright (C) 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html


function varargout = nisp_bruteforcesa ( varargin )
    // Compute sensitivity indices by brute force.
    //
    // Calling Sequence
    //   s=nisp_bruteforcesa(func,nx)
    //   s=nisp_bruteforcesa(func,nx,randgen)
    //   s=nisp_bruteforcesa(func,nx,randgen,n)
    //   [s,nbevalf]=nisp_bruteforcesa(...)
    //
    // Parameters
    //   func : a function or a list, the name of the function to be evaluated. 
    //   nx : a 1-by-1 matrix of floating point integers, the number of inputs of the function.
    //   randgen : a function or a list, the random number generator. (default = uniform random variables)
    //   n : a 1-by-1 matrix of floating point integers (default n=10000), the number of Monte-Carlo experiments, for each sensitivity index
    //   s : a nx-by-1 matrix of doubles, the first order sensitivity indices
    //   nbevalf : a nx-by-1 matrix of doubles, the actual number of function evaluations.
    //
    // Description
    // The algorithm uses the Sobol method to compute the first order 
    // sensitivity indices.
    // This function assumes that the input variables are independent. 
    // By design, the algorithm produces sensitivity indices which are 
    // positive.
    // If a computed indice is greater than 1, it is set to 1.
    // It is not guaranteed that the sum of indices in s is 1: 
    // in fact this is rarely the case.
    //
    // Any optional input argument equal to the empty matrix will be set to its 
    // default value.
    //
    // The function should have header 
    //
    // <screen>
    // y = func ( x ) 
    // </screen>
    //
    // where x is a m-by-nx matrix of doubles, 
    // where m is the number of experiments to perform, 
    // nx is the number of input 
    // random variables, and y is a m-by-1 matrix of doubles.
    //
    // It might happen that the function requires additionnal 
    // arguments to be evaluated.
    // In this case, we can use the following feature.
    // The argument <literal>func</literal> can also be a list, with header 
    //
    // <screen>
    // y = f ( x , a1 , a2 , ... ).
    // </screen>
    //
    // In this case the <literal>func</literal> variable should hold the list (f,a1,a2,...) and the 
    // input arguments a1, a2, will be automatically be appended at the 
    // end of the calling sequence of <literal>f</literal>.
    //
    // The random number generator must have header 
    //
    // <screen>
    // x = randgen ( n , i ) 
    // </screen>
    //
    // where 
    //
    // <itemizedlist>
    // <listitem><para>n is a 1-by-1 matrix of floating point integers, 
    // the number of basic Monte-Carlo experiments</para></listitem>
    // <listitem><para>i is a 1-by-1 matrix of floating point integers 
    // representing the index of the input variable, where i is in 
    // the set {1,2,...,nx}.
    // </para>
    // </listitem>
    // <listitem>
    // <para>
    // x is a n-by-1 matrix of doubles.
    // </para>
    // </listitem>
    // </itemizedlist>
    // On output, x must contain random numbers sampled 
    // from the distribution function associated with the 
    // input variable #i.
    //
    // It might happen that the random number generator requires additionnal 
    // arguments to be evaluated.
    // In this case, we can use the following feature.
    // The argument <literal>randgen</literal> can also be a list, with header 
    //
    // <screen>
    // x = rg ( n , i , a1 , a2 , ... ).
    // </screen>
    //
    // In this case the randgen variable should hold the 
    // list <literal>(rg,a1,a2,...)</literal> and the 
    // input arguments a1, a2, will be automatically be appended at the 
    // end of the calling sequence of <literal>rg</literal>.
    // 
    // Examples
    // // Compute the first order sensitivity indices 
    // // of the ishigami function.
    // // Three random variables uniform in [-pi,pi].
    // function y=ishigami(x)
    //     a=7.
    //     b=0.1
    //     s1=sin(x(:,1))
    //     s2=sin(x(:,2))
    //     x34 = x(:,3).^4
    //     y(:,1) = s1 + a.*s2.^2 + b.*x34.*s1
    // endfunction
    // function x = myrandgen ( n, i)
    //     x = distfun_unifrnd(-%pi,%pi,n,1)
    // endfunction
    // a=7.;
    // b=0.1;
    // exact = nisp_ishigamisa(a,b);
    // n = 500;
    // nx = 3;
    // [s,nbevalf]=nisp_bruteforcesa(ishigami,nx,myrandgen,n);
    // [s(1),exact.S1]
    // [s(2),exact.S2]
    // [s(3),exact.S3]
    //
    // // See the number of significant digits
    // // when N increases
    //stacksize("max");
    //imax=11;
    //for i=2:11
    //    n=2^i;
    //    s=nisp_bruteforcesa(ishigami,nx,myrandgen,n);
    //    d(i-1,1)=assert_computedigits(s(1),exact.S1);
    //    d(i-1,2)=assert_computedigits(s(2),exact.S2);
    //end
    //scf();
    //plot(1:imax-1,d(:,1)',"r-");
    //plot(1:imax-1,d(:,2)',"b-");
    //legend(["S1","S2"]);
    //xtitle("","Log2(N)","Digits");
    //
    // // Repeat the computation N times
    // // See the histogram
    // stacksize("max");
    // n=2^7;
    // nsamples=100;
    // smat=zeros(3,nsamples);
    // for i=1:nsamples
    //     s(:,i)=nisp_bruteforcesa(ishigami,nx,myrandgen,n);
    // end
    // scf();
    // //
    // subplot(1,3,1);
    // histplot(20,s(1,:))
    // plot([exact.S1,exact.S1],[0,10],"r-");
    // xtitle("","S1","Frequency");
    // legend(["Data","Exact"]);
    // //
    // subplot(1,3,2);
    // histplot(20,s(2,:))
    // plot([exact.S2,exact.S2],[0,10],"r-");
    // xtitle("","S2","Frequency");
    // legend(["Data","Exact"]);
    // //
    // subplot(1,3,3);
    // histplot(20,s(3,:))
    // plot([exact.S3,exact.S3],[0,200],"r-");
    // xtitle("","S3","Frequency");
    // legend(["Data","Exact"]);
    //
    // Authors
    // Copyright (C) 2011 - DIGITEO - Michael Baudin
    // Copyright (C) 2013 - Michael Baudin

    function x = getsampling ( nx , n , i , __brute_randgen__ )
        if ( typeof( __brute_randgen__ ) == "list" ) then
            // List
            __brute_randgen__f = __brute_randgen__ (1);
        end
        if ( typeof( __brute_randgen__ ) == "list" ) then
            // List
            x = __brute_randgen__f ( n , i  , __brute_randgen__ (2:$));
        else
            // Macro or compiled macro
            x = __brute_randgen__ ( n , i );
        end

        if ( size(x)<>[n nx] ) then
            errmsg = msprintf(gettext("%s: The size of the output xi of randgen is [%d,%d], which is different from [%d,%d]."), "nisp_bruteforcesa" , size(x,"r") , size(x,"c") , n , nx );
            error(errmsg)
        end
    endfunction

    function x = __nisp_sobol_rgdef__ ( n , i )
        x = unifrnd_(0,1,n,1)
    endfunction

    function y = getfvalue ( x , __brute_func__ )
        if ( typeof( __brute_func__ ) == "list" ) then
            // List
            __brute_func__f = __brute_func__ (1);
            y = __brute_func__f ( x , __brute_func__ (2:$));
        else
            // Macro or compiled macro
            y = __brute_func__ ( x );
        end
    endfunction

    [lhs, rhs] = argn();
    apifun_checkrhs ( "nisp_bruteforcesa" , rhs , 2 : 4 )
    apifun_checklhs ( "nisp_bruteforcesa" , lhs , [0 1 2] )
    //
    __brute_func__ = varargin(1)
    nx = varargin(2)
    __brute_randgen__ = apifun_argindefault ( varargin , 3 , __nisp_sobol_rgdef__ )
    n = apifun_argindefault ( varargin , 4 , 10000 )
    //
    // Check input arguments
    //
    // Check type
    apifun_checktype ( "nisp_bruteforcesa" , __brute_func__ ,   "func" ,  1 , [ "function" "list" ] )
    apifun_checktype ( "nisp_bruteforcesa" , nx ,      "nx" ,     2 , "constant" )
    apifun_checktype ( "nisp_bruteforcesa" , __brute_randgen__ ,  "randgen" , 3 , [ "function" "list" ] )
    apifun_checktype ( "nisp_bruteforcesa" , n ,  "n" , 4 , "constant" )
    //
    // Check size
    apifun_checkscalar ( "nisp_bruteforcesa" , nx ,       "nx" ,       2 )
    apifun_checkscalar ( "nisp_bruteforcesa" , n ,   "n" ,   4 )
    //
    // Check content
    apifun_checkgreq ( "nisp_bruteforcesa" , nx ,  "nx" ,  2 , 1 )
    apifun_checkgreq ( "nisp_bruteforcesa" , n ,   "n" ,   4 , 1 )
    //
    // Proceed...
    // 1. Compute the variance of y.
    // Define a sampling for x
    x=zeros(n,nx)
    for i=1:nx
        x(1:n,i) = getsampling ( nx , n , i, __brute_randgen__ )
    end
    // Perform the experiments
    y = getfvalue ( x , __brute_func__ );
    Ey= mean(y);
    Vy= variance(y);
    //
    // 2. Compute E(y|xi) for several values of xi.
    for i = 1 : nx
        // Get the sampling for i
        x=zeros(n^2,nx)
        for j=1:nx
            if (j==i) then
                xj = getsampling ( nx , n , j, __brute_randgen__ )
                xj=xj.*.ones(n,1)
            else
                xj = getsampling ( nx , n^2 , j, __brute_randgen__ )
            end
            x(1:n^2,j)=xj
        end
        // Perform the experiments
        y = getfvalue ( x , __brute_func__ );
        // Store E(y|xi) in Ey_all(k), for k=1,2,...,n.
        // Each block of n Y-values corresponds to one sample of xi.
        // Reshape y, so that each column correspond to one sample of xi.
        y= matrix(y,n,n);
        Ey_all = mean(y,"r")';
        //
        // Compute Si
        s(i) = variance(Ey_all)/Vy;
        s(i)=min(s(i),1)
    end
    nbevalf = n + nx*n^2
    //
    // Set outputs
    varargout(1) = s
    varargout(2) = nbevalf
endfunction

