// Copyright (C) 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html


function [st , nbevalf] = nisp_sobolsaTotal ( varargin )
    // Compute sensitivity indices by Sobol, Ishigami, Homma.
    //
    // Calling Sequence
    //   st = nisp_sobolsaTotal ( func , nx )
    //   st = nisp_sobolsaTotal ( func , nx , randgen )
    //   st = nisp_sobolsaTotal ( func , nx , randgen , n )
    //   st = nisp_sobolsaTotal ( func , nx , randgen , n , inrange)
    //   [ st , nbevalf ] = nisp_sobolsaTotal ( ... )
    //
    // Parameters
    //   func : a function or a list, the name of the function to be evaluated. 
    //   nx : a 1-by-1 matrix of floating point integers, the number of inputs of the function.
    //   randgen : a function or a list, the random number generator. (default = uniform random variables)
    //   n : a 1-by-1 matrix of floating point integers (default n=10000), the number of Monte-Carlo experiments, for each sensitivity index
    //   inrange : a 1-by-1 matrix of booleans (default inrange = %t), set to true to restrict the sensitivity indices into [0,1].
    //   st : a nx-by-1 matrix of doubles, the total sensitivity indices
    //   nbevalf : a nx-by-1 matrix of doubles, the actual number of function evaluations.
    //
    // Description
    // The algorithm uses the Sobol method to compute the total 
    // sensitivity indices.
    //
    // This method assumes that all the input random variables are 
    // independent.
    //
    // On output, the sensitivity indices are forced to be in the range [0,1].
    //
    // Any optional input argument equal to the empty matrix will be set to its
    // default value.
    //
    // See the <literal>nisp_sobolFirst</literal> function for the management of 
    // <literal>func</literal> and <literal>randgen</literal>.
    //
    // Examples
    // // Compute the total sensitivity indices of the ishigami function.
    // // Three random variables uniform in [-pi,pi].
    // function y = ishigami (x)
    //   a=7.
    //   b=0.1
    //   s1=sin(x(:,1))
    //   s2=sin(x(:,2))
    //   x34 = x(:,3).^4
    //   y(:,1) = s1 + a.*s2.^2 + b.*x34.*s1
    // endfunction
    // function x = myrandgen ( m , i )
    //   x = distfun_unifrnd(-%pi,%pi,m,1)
    // endfunction
    // a=7.;
    // b=0.1;
    // exact = nisp_ishigamisa ( a , b )
    // n = 1000;
    // nx = 3;
    // [ st , nbevalf ] = nisp_sobolsaTotal ( ishigami , nx , myrandgen )
    //
    // Authors
    //   Michael Baudin - 2011 - DIGITEO

    // Load Internals lib
    path = nisp_getpath (  )
    nispinternallib  = lib(fullfile(path,"macros","internals"))


    [lhs, rhs] = argn();
    apifun_checkrhs ( "nisp_sobolsaTotal" , rhs , 2 : 5 )
    apifun_checklhs ( "nisp_sobolsaTotal" , lhs , [0 1 2] )
    //
    [ __sobolfo_func__ , nx , __sobolfo_randgen__ , n , inrange ] = ..
    nisp_sobolsaGetArgs ( "nisp_sobolsaTotal", varargin )
    //
    // Proceed...
    // Create a sampling A
    A = nisp_getsampling ( n , nx , __sobolfo_randgen__ )
    // Create a sampling B
    B = nisp_getsampling ( n , nx , __sobolfo_randgen__ )
    // Perform the experiments in A
    ya = nisp_evalfunc ( A , __sobolfo_func__ )
    //
    for i = 1 : nx
        C(:,1:i-1) = A(:,1:i-1);
        C(:,i)=B(:,i);
        C(:,i+1:nx) = A(:,i+1:nx);
        yc = nisp_evalfunc ( C , __sobolfo_func__ )
        st(i) = 1 - nisp_sensitivityindex ( ya , yc );
    end
	st = nisp_sensitivityTorange(st,inrange)
    nbevalf = n + nx * n
endfunction

